package com.tzld.longarticle.recommend.server.service.recommend;

import cn.hutool.core.collection.CollectionUtil;
import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.JSONArray;
import com.alibaba.fastjson.JSONObject;
import com.ctrip.framework.apollo.spring.annotation.ApolloJsonValue;
import com.google.common.collect.Lists;
import com.tzld.longarticle.recommend.server.common.enums.StatusEnum;
import com.tzld.longarticle.recommend.server.common.enums.recommend.AccountBusinessTypeEnum;
import com.tzld.longarticle.recommend.server.common.enums.recommend.ArticleTypeEnum;
import com.tzld.longarticle.recommend.server.common.enums.recommend.ContentPoolEnum;
import com.tzld.longarticle.recommend.server.common.enums.recommend.RankStrategyEnum;
import com.tzld.longarticle.recommend.server.mapper.aigc.AigcBaseMapper;
import com.tzld.longarticle.recommend.server.mapper.longArticle.LongArticleBaseMapper;
import com.tzld.longarticle.recommend.server.model.dto.ProduceContentDTO;
import com.tzld.longarticle.recommend.server.model.dto.PublishContentDTO;
import com.tzld.longarticle.recommend.server.model.entity.aigc.*;
import com.tzld.longarticle.recommend.server.model.entity.crawler.AccountAvgInfo;
import com.tzld.longarticle.recommend.server.model.entity.crawler.Article;
import com.tzld.longarticle.recommend.server.model.entity.crawler.ArticleDetailInfo;
import com.tzld.longarticle.recommend.server.model.entity.crawler.PublishSortLog;
import com.tzld.longarticle.recommend.server.model.entity.longArticle.ArticlePoolPromotionSource;
import com.tzld.longarticle.recommend.server.model.entity.longArticle.DatastatScore;
import com.tzld.longarticle.recommend.server.model.entity.longArticle.DatastatSortStrategy;
import com.tzld.longarticle.recommend.server.model.param.MiniprogramTaskParam;
import com.tzld.longarticle.recommend.server.model.param.PublishContentParam;
import com.tzld.longarticle.recommend.server.model.vo.FirstContentScoreExport;
import com.tzld.longarticle.recommend.server.model.vo.IntermediateIndicatorsExport;
import com.tzld.longarticle.recommend.server.model.vo.NewSortStrategyExport;
import com.tzld.longarticle.recommend.server.model.vo.aigc.ProduceAuditLayoutContentObjVO;
import com.tzld.longarticle.recommend.server.repository.aigc.*;
import com.tzld.longarticle.recommend.server.repository.crawler.AccountAvgInfoRepository;
import com.tzld.longarticle.recommend.server.repository.crawler.ArticleDetailInfoRepository;
import com.tzld.longarticle.recommend.server.repository.crawler.ArticleRepository;
import com.tzld.longarticle.recommend.server.repository.crawler.PublishSortLogRepository;
import com.tzld.longarticle.recommend.server.repository.longArticle.ArticleCategoryRepository;
import com.tzld.longarticle.recommend.server.repository.longArticle.ArticlePoolPromotionSourceRepository;
import com.tzld.longarticle.recommend.server.repository.longArticle.DatastatScoreRepository;
import com.tzld.longarticle.recommend.server.service.recommend.score.ScoreStrategy;
import com.tzld.longarticle.recommend.server.util.DateUtils;
import com.tzld.longarticle.recommend.server.util.MapBuilder;
import com.tzld.longarticle.recommend.server.util.feishu.FeiShu;
import com.xxl.job.core.biz.model.ReturnT;
import com.xxl.job.core.handler.annotation.XxlJob;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.collections4.CollectionUtils;
import org.springframework.beans.BeanUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.data.util.Pair;
import org.springframework.http.*;
import org.springframework.stereotype.Service;
import org.springframework.util.StringUtils;
import org.springframework.web.client.RestTemplate;

import java.lang.reflect.Field;
import java.util.*;
import java.util.function.Function;
import java.util.stream.Collectors;

@Service
@Slf4j
public class DataDashboardService {

    @Autowired
    private ArticleRepository articleRepository;
    @Autowired
    private ArticleDetailInfoRepository articleDetailInfoRepository;
    @Autowired
    private AccountAvgInfoRepository accountAvgInfoRepository;
    @Autowired
    private PublishSortLogRepository publishSortLogRepository;
    @Autowired
    private LongArticleBaseMapper longArticleBaseMapper;
    @Autowired
    private AigcBaseMapper aigcBaseMapper;
    @Autowired
    private PublishAccountRepository publishAccountRepository;
    @Autowired
    private PublishContentLayOutRepository publishContentLayOutRepository;
    @Autowired
    private PublishPlanRepository publishPlanRepository;
    @Autowired
    private ProducePlanExeRecordRepository producePlanExeRecordRepository;
    @Autowired
    private ProducePlanRepository producePlanRepository;
    @Autowired
    private ProducePlanInputSourceRepository producePlanInputSourceRepository;
    @Autowired
    private DatastatScoreRepository datastatScoreRepository;
    @Autowired
    private ArticleCategoryRepository articleCategoryRepository;
    @Autowired
    private ArticlePoolPromotionSourceRepository articlePoolPromotionSourceRepository;

    @ApolloJsonValue("${export.account.ghId:[]}")
    private static List<String> ghIdList;
    private static final String sheetToken = "M0pLs3uF6hfL0htn2dMcB9eFn8e";

    public void export(String beginDate, String endDate) {
        List<String> dateStrList = DateUtils.getBeforeDays(beginDate, endDate, 5);
        exportFeishuNewSortStrategy(dateStrList, ArticleTypeEnum.QUNFA.getVal(), StatusEnum.ZERO.getCode(),
                sheetToken, "7d4e12");
    }

    @XxlJob("scheduledExport")
    public ReturnT<String> scheduledExport(String param) {
        List<String> dateStrList = DateUtils.getBeforeDays(null, null, 5);
        exportFeishuNewSortStrategy(dateStrList, ArticleTypeEnum.QUNFA.getVal(), StatusEnum.ZERO.getCode(),
                sheetToken, "7d4e12");
        return ReturnT.SUCCESS;
    }

    public void exportWuXianLiu(String beginDate, String endDate) {
        List<String> dateStrList = DateUtils.getBeforeDays(beginDate, endDate, 5);
        exportFeishuNewSortStrategy(dateStrList, ArticleTypeEnum.WUXIANLIU.getVal(), StatusEnum.ONE.getCode(),
                sheetToken, "Ucycvw");
    }

    @XxlJob("scheduledExportWuXianLiu")
    public ReturnT<String> scheduledExportWuXianLiu(String param) {
        List<String> dateStrList = DateUtils.getBeforeDays(null, null, 5);
        exportFeishuNewSortStrategy(dateStrList, ArticleTypeEnum.WUXIANLIU.getVal(), StatusEnum.ONE.getCode(),
                sheetToken, "Ucycvw");
        return ReturnT.SUCCESS;
    }


    private void exportFeishuNewSortStrategy(List<String> dateStrList, String articleType, Integer filter,
                                             String sheetToken, String sheetId) {
        String minDate = dateStrList.stream().min(String::compareTo).orElse("");
        String maxDate = dateStrList.stream().max(String::compareTo).orElse("");
        List<NewSortStrategyExport> newContentsYesData = newSortStrategyData(minDate, maxDate, articleType, filter);
        if (CollectionUtil.isEmpty(newContentsYesData)) {
            return;
        }

        int rowNum = newContentsYesData.size();
        List<List<Object>> rows = new ArrayList<>();
        Field[] fields = NewSortStrategyExport.class.getDeclaredFields();
        for (NewSortStrategyExport datum : newContentsYesData) {
            List<Object> rowDatas = new ArrayList<>();
            rows.add(rowDatas);

            for (Field field : fields) {
                field.setAccessible(true);
                try {
                    rowDatas.add(field.get(datum));
                } catch (IllegalAccessException e) {
                    log.error("获取值出错：{}", field.getName());
                } catch (Exception e) {
                    throw new RuntimeException(e.getMessage());
                }
            }
        }

        List<Pair<String, String>> styles = Arrays
                .asList(
                        Pair.of("AA", "0.00%"),
                        Pair.of("AB", "0.00%"),
                        Pair.of("AC", "0.00%"),
                        Pair.of("AD", "0.00%"),
                        Pair.of("AE", "0.00%"),
                        Pair.of("AF", "0.00%"),
                        Pair.of("AG", "0.00%"),
                        Pair.of("AH", "0.00%"),
                        Pair.of("AI", "0.00%"),
                        Pair.of("AO", "0.00%"),
                        Pair.of("AP", "0.00%"),
                        Pair.of("AQ", "0.00%")
                );

        doSendFeishuSheet(dateStrList, sheetToken, sheetId, rowNum, rows, 2, styles);
    }

    private List<NewSortStrategyExport> newSortStrategyData(String beginDate, String endDate,
                                                            String articleType, Integer filter) {
        long beginTimestamp = DateUtils.dateStrToTimestamp(beginDate, "yyyyMMdd");
        long endTimestamp = DateUtils.dateStrToTimestamp(endDate, "yyyyMMdd") + 86400;
        List<AccountAvgInfo> accountAvgInfoList = accountAvgInfoRepository.findAll();
        log.info("newSortStrategyData accountAvgInfoList finish");
        List<String> ghIds = accountAvgInfoList.stream().map(AccountAvgInfo::getGhId).distinct().collect(Collectors.toList());

        List<Article> articleList = articleRepository.getByGhIdInAndPublishTimestampBetweenAndTypeEquals(ghIds,
                beginTimestamp, endTimestamp, articleType);
        Map<String, Map<String, Map<Integer, Article>>> articleMap = articleList.stream().collect(Collectors.groupingBy(Article::getGhId,
                Collectors.groupingBy(Article::getAppMsgId, Collectors.toMap(Article::getItemIndex, o -> o))));
        log.info("newSortStrategyData articleList finish");
        Set<String> snList = articleList.stream().map(Article::getWxSn).collect(Collectors.toSet());
        Map<String, Article> wxSnMap = articleList.stream().collect(Collectors.toMap(Article::getWxSn, Function.identity()));
        List<ArticleDetailInfo> articleDetailInfoList = new ArrayList<>();
        for (List<String> partitions : Lists.partition(new ArrayList<>(snList), 1000)) {
            articleDetailInfoList.addAll(articleDetailInfoRepository.getAllByWxSnIn(partitions));
        }
        log.info("newSortStrategyData articleDetailInfoList finish");
        Map<String, List<ArticleDetailInfo>> articleDetailInfoMap = articleDetailInfoList.stream()
                .collect(Collectors.groupingBy(ArticleDetailInfo::getWxSn));
        List<PublishSortLog> sortLogList = publishSortLogRepository.findByGhIdInAndDateStrBetween(ghIds, beginDate, endDate);
        log.info("newSortStrategyData sortLogList finish");
        Map<String, Map<String, Map<String, PublishSortLog>>> sortStrategyMap = sortLogList.stream()
                .collect(Collectors.groupingBy(PublishSortLog::getGhId,
                        Collectors.groupingBy(PublishSortLog::getDateStr, Collectors.toMap(PublishSortLog::getTitle,
                                Function.identity(), (existing, replacement) -> existing))));
        Map<String, Map<String, Map<String, AccountAvgInfo>>> accountAvgInfoIndexMap = accountAvgInfoList.stream()
                .filter(o -> Objects.nonNull(o.getReadAvg()) && o.getReadAvg() > 0 && o.getFans() > 1000)
                .collect(Collectors.groupingBy(AccountAvgInfo::getGhId, Collectors.groupingBy(AccountAvgInfo::getUpdateTime,
                        Collectors.toMap(AccountAvgInfo::getPosition, o -> o))));
        // 获取发布账号
        List<PublishAccount> publishAccountList = publishAccountRepository.getAllByGhIdIn(ghIds);
        log.info("newSortStrategyData publishAccountList finish");
        Map<String, PublishAccount> publishAccountMap = publishAccountList.stream().collect(Collectors.toMap(PublishAccount::getGhId, o -> o));
        // 获取发布内容
        List<PublishContentParam> publishContentParamList = articleList.stream().map(article -> {
            PublishContentParam item = new PublishContentParam();
            item.setTitle(article.getTitle());
            PublishAccount account = publishAccountMap.get(article.getGhId());
            if (Objects.nonNull(account)) {
                item.setPublishAccountId(account.getId());
                return item;
            }
            return null;
        }).filter(Objects::nonNull).collect(Collectors.toList());
        List<PublishContentDTO> publishContents = new ArrayList<>();
        for (List<PublishContentParam> partitions : Lists.partition(publishContentParamList, 100)) {
            publishContents.addAll(aigcBaseMapper.getPublishContentByTitle(partitions));
        }
        log.info("newSortStrategyData publishContents finish");
        Map<String, Map<String, Map<Long, PublishContentDTO>>> publishContentMap = publishContents.stream()
                .filter(o -> Objects.nonNull(o.getPublishTimestamp()))
                .sorted(Comparator.comparingLong(PublishContentDTO::getPublishTimestamp)).collect(
                        Collectors.groupingBy(PublishContentDTO::getPublishAccountId,
                                Collectors.groupingBy(PublishContentDTO::getTitle,
                                        Collectors.toMap(PublishContentDTO::getPublishTimestamp, o -> o,
                                                (existing, replacement) -> replacement))));
        // 获取发布内容排版
        List<String> publishContentIds = publishContents.stream().map(PublishContentDTO::getId).collect(Collectors.toList());
        List<PublishContentLayout> publishContentLayoutList = new ArrayList<>();
        for (List<String> partitions : Lists.partition(publishContentIds, 1000)) {
            publishContentLayoutList.addAll(publishContentLayOutRepository.findByPublishContentIdIn(partitions));
        }
        log.info("newSortStrategyData publishContentLayoutList finish");
        Map<String, PublishContentLayout> publishContentLayoutMap = publishContentLayoutList.stream()
                .collect(Collectors.toMap(PublishContentLayout::getPublishContentId, o -> o,
                        (existing, replacement) -> replacement));
        //获取发布计划
        List<String> publishPlanIds = publishContents.stream().map(PublishContentDTO::getPlanId).distinct()
                .collect(Collectors.toList());
        List<PublishPlan> publishPlanList = publishPlanRepository.findByIdIn(publishPlanIds);
        log.info("newSortStrategyData publishPlanList finish");
        Map<String, PublishPlan> publishPlanMap = publishPlanList.stream()
                .collect(Collectors.toMap(PublishPlan::getId, o -> o));
        // 获取生成记录
        List<String> contentSourceIds = publishContents.stream()
                .filter(o -> Arrays.asList(1, 2).contains(o.getSourceType()))
                .map(PublishContentDTO::getSourceId).distinct().collect(Collectors.toList());
        List<ProducePlanExeRecord> planExeRecordList = new ArrayList<>();
        for (List<String> partitions : Lists.partition(contentSourceIds, 1000)) {
            planExeRecordList.addAll(producePlanExeRecordRepository.findByPlanExeIdIn(partitions));
        }
        log.info("newSortStrategyData planExeRecordList finish");
        Map<String, ProducePlanExeRecord> planExeRecordMap = planExeRecordList.stream()
                .collect(Collectors.toMap(ProducePlanExeRecord::getPlanExeId, o -> o));
        // 获取生成计划
        List<String> producePlanIds = planExeRecordList.stream().map(ProducePlanExeRecord::getPlanId).distinct()
                .collect(Collectors.toList());
        List<ProducePlan> producePlanList = producePlanRepository.findByIdIn(producePlanIds);
        log.info("newSortStrategyData producePlanList finish");
        Map<String, ProducePlan> producePlanMap = producePlanList.stream()
                .collect(Collectors.toMap(ProducePlan::getId, o -> o));
        // 获取生成计划输入
        List<ProducePlanInputSource> inputSourceList = producePlanInputSourceRepository.findByPlanIdIn(producePlanIds);
        log.info("newSortStrategyData inputSourceList finish");
        Map<String, List<ProducePlanInputSource>> inputSourceMap = inputSourceList.stream()
                .collect(Collectors.groupingBy(ProducePlanInputSource::getPlanId));
        // 获取抓取内容关联
        List<String> crawlerChannelContentIds = publishContents.stream().map(PublishContentDTO::getCrawlerChannelContentId)
                .distinct().collect(Collectors.toList());
        List<CrawlerPlanResultRel> resultRelList = aigcBaseMapper.getCrawlerPlanRelByChannelContentIds(crawlerChannelContentIds);
        log.info("newSortStrategyData resultRelList finish");
        Map<String, List<CrawlerPlanResultRel>> resultRelMap = resultRelList.stream()
                .collect(Collectors.groupingBy(CrawlerPlanResultRel::getChannelSourceId));
        // 获取抓取计划
        List<String> crawlerPlanIds = resultRelList.stream().map(CrawlerPlanResultRel::getPlanId).distinct()
                .collect(Collectors.toList());
        List<CrawlerPlan> crawlerPlanList = aigcBaseMapper.getCrawlerPlanByPlanIds(crawlerPlanIds);
        log.info("newSortStrategyData crawlerPlanList finish");
        Map<String, CrawlerPlan> crawlerPlanMap = crawlerPlanList.stream()
                .collect(Collectors.toMap(CrawlerPlan::getId, o -> o));
        // 获取小程序任务
        List<MiniprogramTaskParam> miniprogramTaskParamList = new ArrayList<>();
        Set<String> distinct = new HashSet<>();
        for (PublishContentDTO publishContent : publishContents) {
            String key = publishContent.getPlanId() + publishContent.getPublishAccountId();
            if (distinct.contains(key)) {
                continue;
            } else {
                distinct.add(key);
            }
            MiniprogramTaskParam param = new MiniprogramTaskParam();
            param.setAccountId(publishContent.getPublishAccountId());
            param.setPlanId(publishContent.getPlanId());
            miniprogramTaskParamList.add(param);
        }
        List<PublishPlanMiniprogramTask> miniprogramTaskList = new ArrayList<>();
        for (List<MiniprogramTaskParam> partitions : Lists.partition(miniprogramTaskParamList, 1000)) {
            miniprogramTaskList.addAll(aigcBaseMapper.getMiniProgramTask(partitions));
        }
        log.info("newSortStrategyData miniprogramTaskList finish");
        Map<String, List<PublishPlanMiniprogramTask>> miniprogramTaskMap = miniprogramTaskList.stream()
                .collect(Collectors.groupingBy(PublishPlanMiniprogramTask::getPlanId));
        // 源生成计划
        List<String> titleList = articleList.stream().map(Article::getTitle).distinct().collect(Collectors.toList());
        Map<String, ProducePlan> sourceTitlePlanMap = getTitleSourceProducePlanMap(titleList);
        log.info("newSortStrategyData sourceTitlePlan finish");
        // 历史发布情况
        List<String> titleMd5List = articleList.stream().map(Article::getTitleMd5).distinct().collect(Collectors.toList());
        List<Article> hisArticleList = new ArrayList<>();
        List<List<String>> titleMd5Partition = Lists.partition(new ArrayList<>(titleMd5List), 1000);
        for (List<String> titleMd5s : titleMd5Partition) {
            hisArticleList.addAll(articleRepository.getByTitleMd5InAndTypeEqualsAndStatusEquals(titleMd5s, articleType, 1));
        }
        log.info("newSortStrategyData hisArticleList finish");
        Map<String, List<Article>> hisArticleMap = hisArticleList.stream().collect(Collectors.groupingBy(Article::getTitle));
        Set<String> hisWxSnList = hisArticleList.stream().map(Article::getWxSn).collect(Collectors.toSet());
        List<ArticleDetailInfo> hisArticleDetailInfoList = new ArrayList<>();
        List<List<String>> hisSnPartition = Lists.partition(new ArrayList<>(hisWxSnList), 1000);
        for (List<String> sns : hisSnPartition) {
            hisArticleDetailInfoList.addAll(articleDetailInfoRepository.getAllByWxSnIn(sns));
        }
        log.info("newSortStrategyData hisArticleDetailInfoList finish");
        Map<String, List<ArticleDetailInfo>> hisArticleDetailInfoMap = hisArticleDetailInfoList.stream()
                .collect(Collectors.groupingBy(ArticleDetailInfo::getWxSn));

        // result
        List<NewSortStrategyExport> result = new ArrayList<>();
        for (Article article : articleList) {
            NewSortStrategyExport obj = new NewSortStrategyExport();
            List<ArticleDetailInfo> articleDetailInfos = articleDetailInfoMap.get(article.getWxSn());
            setObjArticleDetailInfo(article, obj, articleDetailInfos);
            Article firstArticle = articleMap.get(article.getGhId()).get(article.getAppMsgId()).get(1);
            Map<String, Map<String, PublishSortLog>> dateStrategy = sortStrategyMap.get(article.getGhId());
            AccountAvgInfo avgInfo = getAccountAvgInfo(accountAvgInfoIndexMap, article.getGhId(),
                    article.getPublishTimestamp(), article.getItemIndex());
            AccountAvgInfo firstAvgInfo = getAccountAvgInfo(accountAvgInfoIndexMap, article.getGhId(),
                    article.getPublishTimestamp(), 1);
            String date = DateUtils.timestampToYMDStr(article.getPublishTimestamp(), "yyyyMMdd");
            setObjBaseInfo(article, obj, date);
            if (Objects.nonNull(firstArticle)) {
                obj.setFirstViewCount(firstArticle.getShowViewCount());
            }
            if (Objects.nonNull(firstAvgInfo)) {
                obj.setFirstAvgViewCount(firstAvgInfo.getReadAvg());
            }
            if (Objects.nonNull(dateStrategy)) {
                Map<String, PublishSortLog> titleStrategyMap = dateStrategy.get(date);
                if (Objects.nonNull(titleStrategyMap)) {
                    PublishSortLog sortLog = titleStrategyMap.get(article.getTitle());
                    if (Objects.nonNull(sortLog)) {
                        String strategy = sortLog.getStrategy();
                        if (!StringUtils.hasText(sortLog.getStrategy())) {
                            if (Objects.equals(articleType, ArticleTypeEnum.WUXIANLIU.getVal())) {
                                strategy = RankStrategyEnum.INFINITE_STRATEGY.getStrategy();
                            } else {
                                for (Map.Entry<String, PublishSortLog> entry : titleStrategyMap.entrySet()) {
                                    strategy = entry.getValue().getStrategy();
                                    break;
                                }
                            }
                        }
                        obj.setStrategy(strategy);
                    }
                }
            }
            if (!StringUtils.hasText(obj.getStrategy()) &&
                    Objects.equals(articleType, ArticleTypeEnum.WUXIANLIU.getVal())) {
                obj.setStrategy(RankStrategyEnum.INFINITE_STRATEGY.getStrategy());
            }
            setObjAvgInfo(article, obj, avgInfo);
            setObjHisRateInfo(article, obj, hisArticleMap, accountAvgInfoIndexMap, hisArticleDetailInfoMap);
            // aigc 数据
            setObjAigcInfo(article, obj, date, publishAccountMap, publishContentMap, publishContentLayoutMap,
                    publishPlanMap, miniprogramTaskMap, planExeRecordMap, producePlanMap, inputSourceMap,
                    resultRelMap, crawlerPlanMap, sourceTitlePlanMap);
            result.add(obj);
        }
        log.info("newSortStrategyData buildData finish");
        if (CollectionUtil.isNotEmpty(result) && filter == StatusEnum.ONE.getCode()) {
            result = result.stream()
                    .filter(o -> Objects.nonNull(o.getPublishMiniProgramNum()) && o.getPublishMiniProgramNum() > 0)
                    .collect(Collectors.toList());
        }
        result.sort(Comparator.comparing(NewSortStrategyExport::getDateStr).reversed()
                .thenComparing(NewSortStrategyExport::getGhId).thenComparing(NewSortStrategyExport::getPosition));
        if (CollectionUtils.isNotEmpty(result)) {
            longArticleBaseMapper.deleteByDateStrBetween(beginDate, endDate, articleType);
            List<DatastatSortStrategy> saveList = new ArrayList<>();
            for (NewSortStrategyExport newSortStrategyExport : result) {
                DatastatSortStrategy item = new DatastatSortStrategy();
                BeanUtils.copyProperties(newSortStrategyExport, item);
                item.setType(articleType);
                PublishAccount publishAccount = publishAccountMap.get(item.getGhId());
                item.setAccountCreateTimestamp(publishAccount.getCreateTimestamp() / 1000);
                Article article = wxSnMap.get(item.getWxSn());
                item.setPublishTimestamp(article.getPublishTimestamp());
                Map<String, Map<String, PublishSortLog>> dateStrategy = sortStrategyMap.get(article.getGhId());
                if (Objects.nonNull(dateStrategy)) {
                    Map<String, PublishSortLog> titleStrategyMap = dateStrategy.get(newSortStrategyExport.getDateStr());
                    if (Objects.nonNull(titleStrategyMap)) {
                        PublishSortLog sortLog = titleStrategyMap.get(article.getTitle());
                        if (Objects.nonNull(sortLog)) {
                            item.setSourceType(sortLog.getSourceType());
                            item.setSourceId(sortLog.getSourceId());
                        }
                    }
                }
                saveList.add(item);
            }
            for (List<DatastatSortStrategy> saveListPartition : Lists.partition(saveList, 1000)) {
                longArticleBaseMapper.batchInsertDatastatSortStrategy(saveListPartition);
            }
        }
        log.info("newSortStrategyData finish");
        return result;
    }

    private void setObjAigcInfo(Article article, NewSortStrategyExport obj, String date,
                                Map<String, PublishAccount> publishAccountMap,
                                Map<String, Map<String, Map<Long, PublishContentDTO>>> publishContentMap,
                                Map<String, PublishContentLayout> publishContentLayoutMap,
                                Map<String, PublishPlan> publishPlanMap,
                                Map<String, List<PublishPlanMiniprogramTask>> miniprogramTaskMap,
                                Map<String, ProducePlanExeRecord> planExeRecordMap,
                                Map<String, ProducePlan> producePlanMap,
                                Map<String, List<ProducePlanInputSource>> inputSourceMap,
                                Map<String, List<CrawlerPlanResultRel>> resultRelMap,
                                Map<String, CrawlerPlan> crawlerPlanMap,
                                Map<String, ProducePlan> sourceTitlePlanMap) {
        PublishAccount publishAccount = publishAccountMap.get(article.getGhId());
        Map<String, Map<Long, PublishContentDTO>> titleContentMap = publishContentMap.get(publishAccount.getId());
        if (Objects.isNull(titleContentMap)) {
            return;
        }
        Map<Long, PublishContentDTO> publishTimeContentMap = titleContentMap.get(article.getTitle());
        if (Objects.isNull(publishTimeContentMap) || publishTimeContentMap.isEmpty()) {
            return;
        }
        PublishContentDTO publishContent = null;
        List<String> hisPublishTimeStrList = publishTimeContentMap.keySet().stream()
                .map(o -> DateUtils.timestampToYMDStr(o / 1000, "yyyyMMdd")).collect(Collectors.toList());
        String publishTime = DateUtils.findNearestDate(hisPublishTimeStrList, date, "yyyyMMdd");
        for (Map.Entry<Long, PublishContentDTO> entry : publishTimeContentMap.entrySet()) {
            String str = DateUtils.timestampToYMDStr(entry.getKey() / 1000, "yyyyMMdd");
            if (publishTime.equals(str)) {
                publishContent = entry.getValue();
            }
        }
        if (Objects.isNull(publishContent)) {
            return;
        }
        PublishContentLayout layout = publishContentLayoutMap.get(publishContent.getId());
        List<ProduceAuditLayoutContentObjVO> layoutList = JSON.parseArray(layout.getContent(), ProduceAuditLayoutContentObjVO.class);
        if (CollectionUtil.isNotEmpty(layoutList)) {
            long miniProgramNum = layoutList.stream().filter(o -> o.getType() == 4).count();
            obj.setPublishMiniProgramNum((int) miniProgramNum);
        }
        PublishPlan publishPlan = publishPlanMap.get(publishContent.getPlanId());
        obj.setPublishPlanName(publishPlan.getName());
        // 插入小程序任务
        List<PublishPlanMiniprogramTask> miniprogramTasks = miniprogramTaskMap.get(publishPlan.getId());
        if (CollectionUtil.isNotEmpty(miniprogramTasks)) {
            miniprogramTasks = miniprogramTasks.stream()
                    .filter(o -> o.getAccountIds().contains(publishAccount.getId()))
                    .collect(Collectors.toList());
            if (CollectionUtil.isNotEmpty(miniprogramTasks)) {
                obj.setPublishMiniProgramInsertStrategy(miniprogramTasks.get(0).getStrategy());
                obj.setPublishMiniProgramInsertUseType(miniprogramTasks.get(0).getMiniprogramUseType() == 1 ? "自然流" : "投流");
            }
        }
        List<ProducePlanInputSource> producePlanInputSourceList = new ArrayList<>();
        if (Arrays.asList(1, 2).contains(publishContent.getSourceType())) {
            ProducePlanExeRecord record = planExeRecordMap.get(publishContent.getSourceId());
            if (Objects.nonNull(record)) {
                ProducePlan producePlan = producePlanMap.get(record.getPlanId());
                obj.setProducePlanName(producePlan.getName());
                obj.setProducePlanTag(producePlan.getPlanTag());
            }
            producePlanInputSourceList = inputSourceMap.get(record.getPlanId());
        }
        List<CrawlerPlanResultRel> crawlerPlanRelList = resultRelMap.get(publishContent.getCrawlerChannelContentId());
        if (CollectionUtil.isNotEmpty(crawlerPlanRelList) && CollectionUtil.isNotEmpty(producePlanInputSourceList)) {
            List<String> inputSourceValues = producePlanInputSourceList.stream()
                    .map(ProducePlanInputSource::getInputSourceValue).collect(Collectors.toList());
            List<CrawlerPlan> crawlerPlanItemList = new ArrayList<>();
            for (CrawlerPlanResultRel crawlerPlanResultRel : crawlerPlanRelList) {
                crawlerPlanItemList.add(crawlerPlanMap.get(crawlerPlanResultRel.getPlanId()));
            }
            for (CrawlerPlan crawlerPlan : crawlerPlanItemList) {
                if (inputSourceValues.contains(crawlerPlan.getId())) {
                    obj.setCrawlerPlanName(crawlerPlan.getName());
                    obj.setCrawlerPlanTag(crawlerPlan.getPlanTag());
                    break;
                }
            }
        }
        ProducePlan sourceProducePlan = sourceTitlePlanMap.get(article.getTitle());
        if (Objects.nonNull(sourceProducePlan)) {
            obj.setSourceProducePlanName(sourceProducePlan.getName());
        }
    }

    private void setObjHisRateInfo(Article article, NewSortStrategyExport obj,
                                   Map<String, List<Article>> hisArticleMap,
                                   Map<String, Map<String, Map<String, AccountAvgInfo>>> accountAvgInfoIndexMap,
                                   Map<String, List<ArticleDetailInfo>> hisArticleDetailInfoMap) {
        List<Article> hisArticles = hisArticleMap.getOrDefault(article.getTitle(), new ArrayList<>());
        hisArticles = hisArticles.stream().filter(o -> o.getPublishTimestamp() < (article.getPublishTimestamp() - 3600 * 8)
                && (1 == o.getItemIndex() || 2 == o.getItemIndex())
                && !ScoreStrategy.hisContentLateFilter(o.getPublishTimestamp())).collect(Collectors.toList());
        Integer readCount = 0;
        Double readAvgCount = 0.0;
        int firstCount = 0;
        int fission0Count = 0;
        for (Article hisArticle : hisArticles) {
            readCount += hisArticle.getShowViewCount();
            AccountAvgInfo accountAvgInfo = getAccountAvgInfo(accountAvgInfoIndexMap, hisArticle.getGhId(),
                    hisArticle.getPublishTimestamp(), hisArticle.getItemIndex());
            if (Objects.nonNull(accountAvgInfo)) {
                readAvgCount += accountAvgInfo.getReadAvg();
            }
            List<ArticleDetailInfo> hisADIList = hisArticleDetailInfoMap.get(hisArticle.getWxSn());
            if (CollectionUtil.isNotEmpty(hisADIList)) {
                Date hisMinDate = hisADIList.stream().map(ArticleDetailInfo::getRecallDt).min(Date::compareTo).orElse(new Date());
                for (ArticleDetailInfo articleDetailInfo : hisADIList) {
                    if (articleDetailInfo.getRecallDt().equals(hisMinDate)) {
                        if (Objects.nonNull(articleDetailInfo.getFirstLevel())) {
                            firstCount += articleDetailInfo.getFirstLevel();
                        }
                        if (Objects.nonNull(articleDetailInfo.getFission0())) {
                            fission0Count += articleDetailInfo.getFission0();
                        }
                    }
                }
            }
        }
        if (readAvgCount > 0) {
            obj.setHisReadRate(readCount / readAvgCount);
        }
        if (readCount > 0) {
            obj.setHisFirstReadRate(firstCount / (double) readCount);
        }
        if (firstCount > 0) {
            obj.setHisFission0FirstRate(fission0Count / (double) firstCount);
        }
    }

    private void setObjBaseInfo(Article article, NewSortStrategyExport obj, String date) {
        obj.setGhId(article.getGhId());
        obj.setAccountName(article.getAccountName());
        obj.setTitle(article.getTitle());
        obj.setLink(article.getContentUrl());
        obj.setPosition(article.getItemIndex());
        obj.setViewCount(article.getShowViewCount());
        obj.setDateStr(date);
        obj.setPublishTime(DateUtils.timestampToYMDStr(article.getPublishTimestamp(), "HH:mm:ss"));
        obj.setWxSn(article.getWxSn());
    }

    private void setObjArticleDetailInfo(Article article, NewSortStrategyExport obj, List<ArticleDetailInfo> articleDetailInfos) {
        if (CollectionUtils.isNotEmpty(articleDetailInfos)) {
            Date publishDate = DateUtils.getStartDateOfDay(article.getPublishTimestamp());
            articleDetailInfos = articleDetailInfos.stream().filter(o -> !o.getRecallDt().before(publishDate))
                    .collect(Collectors.toList());
            Date minDate = articleDetailInfos.stream().map(ArticleDetailInfo::getRecallDt).min(Date::compareTo).orElse(new Date());
            Date secondDate = articleDetailInfos.stream().map(ArticleDetailInfo::getRecallDt).filter(o -> o.after(minDate)).min(Date::compareTo).orElse(new Date());
            Date thirdDate = articleDetailInfos.stream().map(ArticleDetailInfo::getRecallDt).filter(o -> o.after(secondDate)).min(Date::compareTo).orElse(new Date());
            int sumFirstLevel = 0;
            int sumFission0 = 0;
            int sumFission1 = 0;
            int sumFission2 = 0;
            int sumSecondFirstLevel = 0;
            int sumSecondFission0 = 0;
            int sumSecondFission1 = 0;
            int sumSecondFission2 = 0;
            int sumThirdFirstLevel = 0;
            int sumThirdFission0 = 0;
            int sumThirdFission1 = 0;
            int sumThirdFission2 = 0;
            for (ArticleDetailInfo articleDetailInfo : articleDetailInfos) {
                if (articleDetailInfo.getRecallDt().equals(minDate)) {
                    sumFirstLevel += Optional.ofNullable(articleDetailInfo.getFirstLevel()).orElse(0);
                    sumFission0 += Optional.ofNullable(articleDetailInfo.getFission0()).orElse(0);
                    sumFission1 += Optional.ofNullable(articleDetailInfo.getFission1()).orElse(0);
                    sumFission2 += Optional.ofNullable(articleDetailInfo.getFission2()).orElse(0);
                }
                if (articleDetailInfo.getRecallDt().equals(secondDate)) {
                    sumSecondFirstLevel += Optional.ofNullable(articleDetailInfo.getFirstLevel()).orElse(0);
                    sumSecondFission0 += Optional.ofNullable(articleDetailInfo.getFission0()).orElse(0);
                    sumSecondFission1 += Optional.ofNullable(articleDetailInfo.getFission1()).orElse(0);
                    sumSecondFission2 += Optional.ofNullable(articleDetailInfo.getFission2()).orElse(0);
                }
                if (articleDetailInfo.getRecallDt().equals(thirdDate)) {
                    sumThirdFirstLevel += Optional.ofNullable(articleDetailInfo.getFirstLevel()).orElse(0);
                    sumThirdFission0 += Optional.ofNullable(articleDetailInfo.getFission0()).orElse(0);
                    sumThirdFission1 += Optional.ofNullable(articleDetailInfo.getFission1()).orElse(0);
                    sumThirdFission2 += Optional.ofNullable(articleDetailInfo.getFission2()).orElse(0);
                }
            }
            obj.setFirstLevel(sumFirstLevel);
            obj.setFission0(sumFission0);
            obj.setFission1(sumFission1);
            obj.setFission2(sumFission2);
            obj.setSecondFirstLevel(sumSecondFirstLevel);
            obj.setSecondFission0(sumSecondFission0);
            obj.setSecondFission1(sumSecondFission1);
            obj.setSecondFission2(sumSecondFission2);
            obj.setThirdFirstLevel(sumThirdFirstLevel);
            obj.setThirdFission0(sumThirdFission0);
            obj.setThirdFission1(sumThirdFission1);
            obj.setThirdFission2(sumThirdFission2);
        }
    }

    private void setObjAvgInfo(Article article, NewSortStrategyExport obj, AccountAvgInfo avgInfo) {
        if (Objects.nonNull(avgInfo)) {
            obj.setAccountMode(avgInfo.getAccountMode());
            obj.setAccountSource(avgInfo.getAccountSource());
            obj.setAccountType(avgInfo.getAccountType());
            obj.setAccountStatus(avgInfo.getAccountStatus());
            obj.setBusinessType(AccountBusinessTypeEnum.from(avgInfo.getBusinessType()).getDescription());
            obj.setFans(avgInfo.getFans());
            obj.setAvgViewCount(avgInfo.getReadAvg());
            if (avgInfo.getReadAvg() > 0) {
                obj.setReadRate((article.getShowViewCount() * 1.0) / avgInfo.getReadAvg());
                if (Objects.nonNull(obj.getFission0())) {
                    obj.setFission0ReadAvgRate((obj.getFission0() * 1.0) / avgInfo.getReadAvg());
                    obj.setFission0ReadAvg100Rate((obj.getFission0() * 1.0) / (avgInfo.getReadAvg() + 100));
                    obj.setFission0ReadAvg500Rate((obj.getFission0() * 1.0) / (avgInfo.getReadAvg() + 500));
                    obj.setFission0ReadAvg1000Rate((obj.getFission0() * 1.0) / (avgInfo.getReadAvg() + 1000));
                }
            }
            if (avgInfo.getFans() > 0) {
                obj.setReadFansRate((article.getShowViewCount() * 1.0) / avgInfo.getFans());
            }
        }
        if (Objects.nonNull(obj.getFirstLevel()) && obj.getFirstLevel() > 0 && Objects.nonNull(obj.getFission0())) {
            obj.setFission0FirstRate((obj.getFission0() * 1.0) / obj.getFirstLevel());
        }
        if (article.getShowViewCount() > 0 && Objects.nonNull(obj.getFirstLevel())) {
            obj.setFirstReadRate((obj.getFirstLevel() * 1.0) / article.getShowViewCount());
        }
        if (Objects.nonNull(obj.getFission0()) && obj.getFission0() > 0 && Objects.nonNull(obj.getFission1())) {
            obj.setFission1Fission0Rate((obj.getFission1() * 1.0) / obj.getFission0());
        }
    }

    private Map<String, ProducePlan> getTitleSourceProducePlanMap(List<String> titleList) {
        List<ProduceContentDTO> sourceProduceContentList = new ArrayList<>();
        for (List<String> partitions : Lists.partition(titleList, 1000)) {
            sourceProduceContentList.addAll(aigcBaseMapper.getSourceProduceContentByTitles(partitions));
        }
        Map<String, List<ProduceContentDTO>> sourceProduceContentMap = sourceProduceContentList.stream()
                .collect(Collectors.groupingBy(ProduceContentDTO::getTitle));
        Map<String, String> sourceTitlePlanIdMap = new HashMap<>();
        List<String> sourceProducePlanIds = new ArrayList<>();
        for (Map.Entry<String, List<ProduceContentDTO>> entry : sourceProduceContentMap.entrySet()) {
            String title = entry.getKey();
            List<ProduceContentDTO> contentList = entry.getValue();
            contentList.sort(Comparator.comparing(ProduceContentDTO::getProduceTimestamp));
            sourceTitlePlanIdMap.put(title, contentList.get(0).getProducePlanId());
            sourceProducePlanIds.add(contentList.get(0).getProducePlanId());
        }
        List<ProducePlan> sourceProducePlanList = producePlanRepository.findByIdIn(sourceProducePlanIds);
        Map<String, ProducePlan> sourceProducePlanMap = sourceProducePlanList.stream()
                .collect(Collectors.toMap(ProducePlan::getId, o -> o));
        Map<String, ProducePlan> sourceTitlePlanMap = new HashMap<>();
        for (Map.Entry<String, String> entry : sourceTitlePlanIdMap.entrySet()) {
            String title = entry.getKey();
            String planId = entry.getValue();
            ProducePlan plan = sourceProducePlanMap.get(planId);
            sourceTitlePlanMap.put(title, plan);
        }
        return sourceTitlePlanMap;
    }


    private static void doSendFeishuSheet(List<String> dateStrList, String sheetToken, String sheetId,
                                          int rowNum, List<List<Object>> rows, Integer startRowIndex,
                                          List<Pair<String, String>> styles) {
        Pair<String, Integer> token = FeiShu.requestAccessToken();
        RestTemplate restTemplate = new RestTemplate();
        HttpHeaders httpHeaders = new HttpHeaders();
        httpHeaders.add("Authorization", "Bearer " + token.getFirst());

        // 先删除掉已存在的dateStr数据
        HttpEntity<Object> queryEntity = new HttpEntity<>(httpHeaders);
        int deleteRowNum = rowNum < 20 ? startRowIndex + dateStrList.size() * rowNum : startRowIndex + (rowNum * 2);
        ResponseEntity<String> queryResponseEntity = restTemplate.exchange(
                String.format("https://open.feishu.cn/open-apis/sheets/v2/spreadsheets/%s/values/%s!A"
                        + startRowIndex + ":A" + deleteRowNum, sheetToken, sheetId),
                HttpMethod.GET, queryEntity, String.class);
        JSONArray values = JSON.parseObject(queryResponseEntity.getBody())
                .getJSONObject("data")
                .getJSONObject("valueRange")
                .getJSONArray("values");
        int count = 0;
        if (!values.isEmpty() && Objects.nonNull(values.get(0)) && !((JSONArray) values.get(0)).isEmpty()
                && Objects.nonNull(((JSONArray) values.get(0)).get(0))
                && dateStrList.contains(((JSONArray) values.get(0)).get(0).toString())) {
            for (Object value : values) {
                if (((JSONArray) value).get(0) != null) {
                    List<String> dates = ((JSONArray) value).stream().map(Object::toString).collect(Collectors.toList());
                    if (dateStrList.contains(dates.get(0))) {
                        count++;
                        continue;
                    }
                }
                break;
            }
        }
        if (count > 0) {
            int delNum = 0;
            do {
                // 删除当前日期已存在的旧数据
                httpHeaders.setContentType(MediaType.APPLICATION_JSON);
                HttpEntity<Object> deleteEntity = new HttpEntity<>(
                        String.format("{\n" +
                                "    \"dimension\": {\n" +
                                "        \"sheetId\": \"%s\",\n" +
                                "        \"majorDimension\": \"ROWS\",\n" +
                                "        \"startIndex\": %s,\n" +
                                "        \"endIndex\": %s\n" +
                                "    }\n" +
                                "}", sheetId, startRowIndex, Math.min(startRowIndex + 4000, count - delNum + startRowIndex) - 1),
                        httpHeaders);
                restTemplate.exchange(String.format("https://open.feishu.cn/open-apis/sheets/v2/spreadsheets/%s/dimension_range", sheetToken),
                        HttpMethod.DELETE, deleteEntity, String.class);
                delNum = Math.min(delNum + 4000, count);
            } while (delNum < count);
        }
        List<List<List<Object>>> partitions = Lists.partition(rows, 1000);
        int startRow = startRowIndex;
        for (List<List<Object>> partition : partitions) {
            // 插入数据
            HttpEntity<Object> postEntity = new HttpEntity<>(MapBuilder
                    .builder()
                    .put("valueRange", MapBuilder
                            .builder()
                            .put("range", String.format("%s!A" + startRow + ":CI", sheetId) + (partition.size() + startRow - 1))
                            .put("values", partition)
                            .build())
                    .build(), httpHeaders);
            ResponseEntity<String> response = restTemplate.exchange(String.format("https://open.feishu.cn/open-apis/sheets/v2/spreadsheets/%s/values_prepend",
                            sheetToken),
                    HttpMethod.POST, postEntity, String.class);
            JSONObject responseJSON = JSONObject.parseObject(response.getBody());
            if (0 != responseJSON.getInteger("code")) {
                log.error("doSendFeishuSheet write error :{}", responseJSON.getString("msg"));
            }
            startRow += partition.size();
        }
//        // 合并单元格
//        if (isMerge) {
//            HttpEntity<Object> mergeEntity = new HttpEntity<>(MapBuilder
//                    .builder()
//                    .put("range", String.format("%s!A" + startRowIndex + ":A", sheetId) + (rowNum + startRowIndex - 1))
//                    .put("mergeType", "MERGE_ALL")
//                    .build(), httpHeaders);
//            restTemplate.exchange(String.format("https://open.feishu.cn/open-apis/sheets/v2/spreadsheets/%s/merge_cells",
//                            sheetToken),
//                    HttpMethod.POST, mergeEntity, String.class);
//        }
        // 此处先简单处理，调整单元格为”百分比小数点“
        try {
            Thread.sleep(1000);
        } catch (Exception e) {
            log.error("export sleep error");
        }
        if (CollectionUtil.isNotEmpty(styles)) {
            startRow = startRowIndex;
            do {
                for (Pair<String, String> style : styles) {
                    HttpEntity<Map<Object, Object>> styleEntity = new HttpEntity<>(MapBuilder
                            .builder()
                            .put("appendStyle",
                                    MapBuilder
                                            .builder()
                                            .put("range", String.format("%s!%s" + startRow + ":%s", sheetId,
                                                    style.getFirst(), style.getFirst())
                                                    + (Math.min(startRow + 4000, rowNum + startRowIndex) - 1))
                                            .put("style",
                                                    MapBuilder
                                                            .builder()
                                                            .put("formatter", style.getSecond())
                                                            .build()
                                            )
                                            .build()
                            )
                            .build(), httpHeaders);
                    restTemplate.exchange(
                            String.format("https://open.feishu.cn/open-apis/sheets/v2/spreadsheets/%s/style",
                                    sheetToken),
                            HttpMethod.PUT,
                            styleEntity,
                            String.class
                    );
                }
                startRow += 4000;
            } while (startRow < rowNum);
        }
    }

    @XxlJob("scheduleExportIntermediateIndicators")
    public ReturnT<String> scheduleIntermediateIndicatorsExport(String param) {
        List<String> dateStrList = DateUtils.getBeforeDays(null, null, 3);
        for (String date : dateStrList) {
            exportFeishuIntermediateIndicators(date, dateStrList, sheetToken, "OuaLWV");
        }
        return ReturnT.SUCCESS;
    }

    public void intermediateIndicatorsExport(String beginDate, String endDate) {
        List<String> dateStrList = DateUtils.getBeforeDays(beginDate, endDate, 3);
        for (String date : dateStrList) {
            exportFeishuIntermediateIndicators(date, dateStrList, sheetToken, "OuaLWV");
        }
    }

    private void exportFeishuIntermediateIndicators(String dateStr, List<String> dateStrList, String sheetToken, String sheetId) {
        List<IntermediateIndicatorsExport> newContentsYesData = intermediateIndicatorsData(dateStr);
        if (CollectionUtil.isEmpty(newContentsYesData)) {
            return;
        }
        long dateStrLong = DateUtils.dateStrToTimestamp(dateStr, "yyyyMMdd");
        List<String> dateStrs = dateStrList.stream().filter(o -> {
            long date = DateUtils.dateStrToTimestamp(o, "yyyyMMdd");
            return date >= dateStrLong;
        }).collect(Collectors.toList());
        int rowNum = newContentsYesData.size();
        List<List<Object>> rows = new ArrayList<>();
        Field[] fields = IntermediateIndicatorsExport.class.getDeclaredFields();
        for (IntermediateIndicatorsExport datum : newContentsYesData) {
            List<Object> rowDatas = new ArrayList<>();
            rows.add(rowDatas);

            for (Field field : fields) {
                field.setAccessible(true);
                try {
                    rowDatas.add(field.get(datum));
                } catch (IllegalAccessException e) {
                    log.error("获取值出错：{}", field.getName());
                } catch (Exception e) {
                    throw new RuntimeException(e.getMessage());
                }
            }
        }

        List<Pair<String, String>> styles = Arrays
                .asList(
                        Pair.of("W", "#,##0.00"),
                        Pair.of("AI", "#,##0.00"),
                        Pair.of("AN", "#,##0.00"),
                        Pair.of("BC", "#,##0.00"),
                        Pair.of("BL", "#,##0.00"),
                        Pair.of("G", "0.00%"),
                        Pair.of("P", "0.00%"),
                        Pair.of("Q", "0.00%"),
                        Pair.of("R", "0.00%"),
                        Pair.of("S", "0.00%"),
                        Pair.of("X", "0.00%"),
                        Pair.of("Y", "0.00%"),
                        Pair.of("Z", "0.00%"),
                        Pair.of("AA", "0.00%"),
                        Pair.of("AF", "0.00%"),
                        Pair.of("AG", "0.00%"),
                        Pair.of("AH", "0.00%"),
                        Pair.of("AO", "0.00%"),
                        Pair.of("AP", "0.00%"),
                        Pair.of("AQ", "0.00%"),
                        Pair.of("AR", "0.00%"),
                        Pair.of("AW", "0.00%"),
                        Pair.of("AX", "0.00%"),
                        Pair.of("AY", "0.00%"),
                        Pair.of("BD", "0.00%"),
                        Pair.of("BE", "0.00%"),
                        Pair.of("BF", "0.00%"),
                        Pair.of("BG", "0.00%"),
                        Pair.of("BM", "0.00%"),
                        Pair.of("BN", "0.00%"),
                        Pair.of("BO", "0.00%"),
                        Pair.of("BP", "0.00%")
                );

        doSendFeishuSheet(dateStrs, sheetToken, sheetId, rowNum, rows, 3, styles);
    }

    public List<IntermediateIndicatorsExport> intermediateIndicatorsData(String dateStr) {
        List<IntermediateIndicatorsExport> result = buildIntermediateIndicatorsData(dateStr);
        Map<String, IntermediateIndicatorsExport> resultMap = result.stream().collect(Collectors.toMap(
                IntermediateIndicatorsExport::getType, o -> o));
        List<String> small = Arrays.asList("gh_084a485e859a", "gh_183d80deffb8", "gh_1b27dd1beeca", "gh_29074b51f2b7",
                "gh_4568b5a7e2fe", "gh_5ff48e9fb9ef", "gh_6d9f36e3a7be", "gh_72bace6b3059", "gh_7b4a5f86d68c",
                "gh_9f8dc5b0c74e", "gh_b676b7ad9b74", "gh_ee78360d06f5", "gh_f25b5fb01977", "gh_f902cea89e48");
        Long dateStart = DateUtils.getStartOfDay(dateStr, "yyyyMMdd");
        Long dateEnd = DateUtils.getEndOfDay(dateStr, "yyyyMMdd");
        long timestamp = DateUtils.dateStrToTimestamp(dateStr, "yyyyMMdd");
        String dateStrS = DateUtils.timestampToYMDStr(timestamp, "yyyy-MM-dd");
        List<AccountAvgInfo> accountAvgInfoList = accountAvgInfoRepository.getAllByUpdateTimeLessThanEqual(dateStrS);
        Map<String, Map<String, Map<String, AccountAvgInfo>>> accountAvgInfoIndexMap = accountAvgInfoList.stream()
                .filter(o -> Objects.nonNull(o.getReadAvg()) && o.getReadAvg() > 0 && o.getFans() > 1000)
                .collect(Collectors.groupingBy(AccountAvgInfo::getGhId, Collectors.groupingBy(AccountAvgInfo::getUpdateTime,
                        Collectors.toMap(AccountAvgInfo::getPosition, o -> o))));
        List<String> ghIds = accountAvgInfoList.stream().map(AccountAvgInfo::getGhId).distinct().collect(Collectors.toList());
        List<Article> articleList = articleRepository.getByGhIdInAndPublishTimestampLessThanAndTypeEquals(ghIds, dateEnd, ArticleTypeEnum.QUNFA.getVal());
        Map<String, Article> articleMap = articleList.stream().collect(Collectors.toMap(Article::getWxSn, o -> o));
        List<Article> todayPublish = articleList.stream().filter(o -> o.getPublishTimestamp() > dateStart).collect(Collectors.toList());
        if (CollectionUtils.isEmpty(todayPublish)) {
            return new ArrayList<>();
        }
        Map<String, List<Article>> hisPublishMap = articleList.stream().collect(Collectors.groupingBy(Article::getTitle));
        List<String> wxSnList = articleList.stream().map(Article::getWxSn).distinct().collect(Collectors.toList());
        List<ArticleDetailInfo> articleDetailInfoList = new ArrayList<>();
        for (List<String> partitions : Lists.partition(wxSnList, 1000)) {
            articleDetailInfoList.addAll(articleDetailInfoRepository.getAllByWxSnIn(partitions));
        }
        Map<String, List<ArticleDetailInfo>> articleDetailInfoMap = articleDetailInfoList.stream()
                .collect(Collectors.groupingBy(ArticleDetailInfo::getWxSn));
        String lessDateStr = DateUtils.getBeforeDayStr(dateStr, "yyyyMMdd", 7);
        List<PublishSortLog> publishSortLogList = publishSortLogRepository.findByDateStrGreaterThanEqual(lessDateStr);
        Map<String, Map<String, Map<Integer, List<PublishSortLog>>>> publishSortLogMap = publishSortLogList.stream()
                .collect(Collectors.groupingBy(PublishSortLog::getGhId,
                        Collectors.groupingBy(PublishSortLog::getDateStr, Collectors.groupingBy(PublishSortLog::getIndex))));
        List<NewSortStrategyExport> newSortStrategyExportList = getNewSortStrategyExportList(todayPublish,
                articleDetailInfoMap, accountAvgInfoIndexMap);

        List<Article> futurePublishList = articleRepository.getByPublishTimestampGreaterThanAndTypeEquals(dateEnd, ArticleTypeEnum.QUNFA.getVal());
        Map<String, List<Article>> futurePublishMap = futurePublishList.stream().collect(Collectors.groupingBy(Article::getTitle));

        List<ArticlePoolPromotionSource> promotionSourceList = articlePoolPromotionSourceRepository.getByStatusAndDeleted(1, 0);
        Map<String, ArticlePoolPromotionSource> promotionSourceMap = promotionSourceList.stream()
                .collect(Collectors.toMap(ArticlePoolPromotionSource::getTitle, Function.identity(), (a, b) -> a));

        Map<String, List<String>> titleTypeMap = new HashMap<>();
        Map<String, Integer> scoreHisPublishTimeMap = new HashMap<>();
        Map<String, Map<Integer, List<String>>> titleTypePoolMap = new HashMap<>();
        Map<String, Set<String>> wxsnHisDistinctSetMap = new HashMap<>();
        Map<String, List<String>> fansAccountTypeMap = new HashMap<>();
        for (NewSortStrategyExport data : newSortStrategyExportList) {
            String type = getArticleType(data, small);
            Article article = articleMap.get(data.getWxSn());
            if (!checkIsAigcPublish(article, publishSortLogMap)) {
                continue;
            }
            IntermediateIndicatorsExport item = resultMap.get(type);
            List<Article> list = hisPublishMap.get(data.getTitle());
            list = list.stream().filter(publish -> publish.getPublishTimestamp() < (article.getPublishTimestamp() - 3600 * 8))
                    .collect(Collectors.toList());
            Integer poolLevel = getArticlePoolLevel(data.getGhId(), list, small);
            // L1层仅统计历史发布在3-8位置文章
            if (poolLevel > 1 && "L1".equals(item.getType())) {
                continue;
            }
            List<String> titles = titleTypeMap.computeIfAbsent(type, k -> new ArrayList<>());
            Map<Integer, List<String>> titlePoolMap = titleTypePoolMap.computeIfAbsent(type, k -> new HashMap<>());
            List<String> poolTitles = titlePoolMap.computeIfAbsent(poolLevel, k -> new ArrayList<>());

            AccountAvgInfo accountAvgInfo = getAccountAvgInfo(accountAvgInfoIndexMap, article.getGhId(),
                    article.getPublishTimestamp(), article.getItemIndex());
//            // L4层仅统计 稳定-大和稳定
//            if ("L4".equals(item.getType()) && (Objects.isNull(accountAvgInfo)
//                    || !StringUtils.hasText(accountAvgInfo.getAccountStatus())
//                    || !accountAvgInfo.getAccountStatus().contains("稳定"))) {
//                continue;
//            }
            // 发布情况
            setPublishSituation(item, type, titleTypeMap, titleTypePoolMap, poolLevel, article, fansAccountTypeMap,
                    accountAvgInfo);
            // 发布表现
            setPublishPerformance(item, data, publishSortLogMap);
            // 发布依赖表现
            setPublishSourcePerformance(item, data, article, accountAvgInfoIndexMap, articleDetailInfoMap, publishSortLogMap,
                    type, scoreHisPublishTimeMap, wxsnHisDistinctSetMap, list, poolLevel, small, hisPublishMap);
            // 发布未来表现
            setPublishFuturePerformance(item, data, poolLevel, promotionSourceMap, futurePublishMap, small);

            titles.add(data.getTitle());
            poolTitles.add(data.getTitle());
        }

        return result;
    }

    private boolean checkIsAigcPublish(Article article,
                                       Map<String, Map<String, Map<Integer, List<PublishSortLog>>>> publishSortLogMap) {
        Map<String, Map<Integer, List<PublishSortLog>>> dateSortMap = publishSortLogMap.get(article.getGhId());
        if (Objects.nonNull(dateSortMap)) {
            for (String dateStr : dateSortMap.keySet()) {
                Map<Integer, List<PublishSortLog>> indexMap = dateSortMap.get(dateStr);
                if (Objects.nonNull(indexMap)) {
                    List<PublishSortLog> logs = indexMap.get(article.getItemIndex());
                    if (CollectionUtil.isNotEmpty(logs)) {
                        for (PublishSortLog log : logs) {
                            if (log.getTitle().equals(article.getTitle())) {
                                return true;
                            }
                        }
                    }
                }
            }
        }
        return false;
    }

    private AccountAvgInfo getAccountAvgInfo(Map<String, Map<String, Map<String, AccountAvgInfo>>> accountAvgInfoIndexMap,
                                             String ghId, Long updateTime, Integer itemIndex) {
        Map<String, Map<String, AccountAvgInfo>> dateAvgMap = accountAvgInfoIndexMap.get(ghId);
        String hisPublishDate = DateUtils.timestampToYMDStr(updateTime, "yyyy-MM-dd");
        AccountAvgInfo avgInfo = null;
        if (Objects.nonNull(dateAvgMap)) {
            List<String> avgMapDateList = new ArrayList<>(dateAvgMap.keySet());
            hisPublishDate = DateUtils.findNearestDate(avgMapDateList, hisPublishDate, "yyyy-MM-dd");
            Map<String, AccountAvgInfo> accountAvgInfoMap = dateAvgMap.get(hisPublishDate);
            if (Objects.nonNull(accountAvgInfoMap)) {
                avgInfo = accountAvgInfoMap.get(itemIndex.toString());
            }
        }
        return avgInfo;
    }

    private void setPublishFuturePerformance(IntermediateIndicatorsExport item, NewSortStrategyExport data, Integer poolLevel,
                                             Map<String, ArticlePoolPromotionSource> promotionSourceMap,
                                             Map<String, List<Article>> futurePublishMap,
                                             List<String> small) {
        int futurePoolLevel = getPromotionPoolLevel(data.getGhId(), data.getTitle(), promotionSourceMap, small);
        if (futurePoolLevel > poolLevel) {
            if (futurePoolLevel == 2) {
                item.setPromotionCountL2(item.getPromotionCountL2() + 1);
                item.setPromotionRateL2((item.getPromotionCountL2() * 1.0) / item.getActualArticleReleaseCount());
            } else if (futurePoolLevel == 3) {
                item.setPromotionCountL3(item.getPromotionCountL3() + 1);
                item.setPromotionRateL3((item.getPromotionCountL3() * 1.0) / item.getActualArticleReleaseCount());
            } else if (futurePoolLevel == 4) {
                item.setPromotionCountL4(item.getPromotionCountL4() + 1);
                item.setPromotionRateL4((item.getPromotionCountL4() * 1.0) / item.getActualArticleReleaseCount());
            }
        }
        List<Article> futurePublishList = futurePublishMap.get(data.getTitle());
        if (CollectionUtils.isEmpty(futurePublishList)) {
            return;
        }
        if (Objects.isNull(data.getSecondFirstLevel())) {
            return;
        }
        item.setT1DayFirstLayerUV(item.getT1DayFirstLayerUV() + data.getSecondFirstLevel());
        item.setT1DayT0FissionUV(item.getT1DayT0FissionUV() + data.getSecondFission0());
        item.setT2DayFirstLayerUV(item.getT2DayFirstLayerUV() + data.getThirdFirstLevel());
        item.setT2DayT0FissionUV(item.getT2DayT0FissionUV() + data.getThirdFission0());
        item.setT1DayAccountArticleRelevanceAvg(item.getAccountArticleRelevanceAvg());
        item.setT2DayAccountArticleRelevanceAvg(item.getAccountArticleRelevanceAvg());
        item.setT1DayViewBase(item.getViewBase());
        item.setT2DayViewBase(item.getViewBase());
        item.setT1DayViewCount(item.getViewCount());
        item.setT2DayViewCount(item.getViewCount());
        double t1DayAverageViewToBaseRatio = item.getT1DayAverageViewToBaseRatio();
        if (data.getViewCount() > 0 && Objects.nonNull(data.getAvgViewCount()) && data.getAvgViewCount() > 0) {
            double t1DayViewToBaseRatio = data.getViewCount() / data.getAvgViewCount();
            t1DayAverageViewToBaseRatio = (t1DayAverageViewToBaseRatio * (item.getActualArticleReleaseCount() - 1)
                    + t1DayViewToBaseRatio) / item.getActualArticleReleaseCount();
        }
        item.setT1DayAverageViewToBaseRatio(t1DayAverageViewToBaseRatio);
        double t1DayAverageMiniAppOpenRate = item.getT1DayAverageMiniAppOpenRate();
        if (data.getSecondFirstLevel() > 0 && Objects.nonNull(data.getAvgViewCount()) && data.getAvgViewCount() > 0) {
            double t1DayMiniAppOpenRate = data.getSecondFirstLevel() / data.getAvgViewCount();
            t1DayAverageMiniAppOpenRate = (t1DayAverageMiniAppOpenRate * (item.getActualArticleReleaseCount() - 1)
                    + t1DayMiniAppOpenRate) / item.getActualArticleReleaseCount();
        }
        item.setT1DayAverageMiniAppOpenRate(t1DayAverageMiniAppOpenRate);
        double t1DayAverageT0FissionRate = item.getT1DayAverageT0FissionRate();
        if (data.getSecondFission0() > 0 && data.getSecondFirstLevel() > 0) {
            double t1DayT0FissionRate = data.getSecondFission0() / (double) data.getSecondFirstLevel();
            t1DayAverageT0FissionRate = (t1DayAverageT0FissionRate * (item.getActualArticleReleaseCount() - 1)
                    + t1DayT0FissionRate) / item.getActualArticleReleaseCount();
        }
        item.setT1DayAverageT0FissionRate(t1DayAverageT0FissionRate);
        double t1DayAverageFirstLayerFissionToViewBaseRatio = item.getT1DayAverageFirstLayerFissionToViewBaseRatio();
        if (data.getSecondFission0() > 0 && Objects.nonNull(data.getAvgViewCount()) && data.getAvgViewCount() > 0) {
            double t1DayFirstLayerFissionToViewBaseRatio = data.getSecondFission0() / data.getAvgViewCount();
            t1DayAverageFirstLayerFissionToViewBaseRatio = (t1DayAverageFirstLayerFissionToViewBaseRatio * (item.getActualArticleReleaseCount() - 1)
                    + t1DayFirstLayerFissionToViewBaseRatio) / item.getActualArticleReleaseCount();
        }
        item.setT1DayAverageFirstLayerFissionToViewBaseRatio(t1DayAverageFirstLayerFissionToViewBaseRatio);
        double t2DayAverageViewToBaseRatio = item.getT2DayAverageViewToBaseRatio();
        if (data.getViewCount() > 0 && Objects.nonNull(data.getAvgViewCount()) && data.getAvgViewCount() > 0) {
            double t2DayViewToBaseRatio = data.getViewCount() / data.getAvgViewCount();
            t2DayAverageViewToBaseRatio = (t2DayAverageViewToBaseRatio * (item.getActualArticleReleaseCount() - 1)
                    + t2DayViewToBaseRatio) / item.getActualArticleReleaseCount();
        }
        item.setT2DayAverageViewToBaseRatio(t2DayAverageViewToBaseRatio);
        double t2DayAverageMiniAppOpenRate = item.getT2DayAverageMiniAppOpenRate();
        if (data.getThirdFirstLevel() > 0 && Objects.nonNull(data.getAvgViewCount()) && data.getAvgViewCount() > 0) {
            double t2DayMiniAppOpenRate = data.getThirdFirstLevel() / data.getAvgViewCount();
            t2DayAverageMiniAppOpenRate = (t2DayAverageMiniAppOpenRate * (item.getActualArticleReleaseCount() - 1)
                    + t2DayMiniAppOpenRate) / item.getActualArticleReleaseCount();
        }
        item.setT2DayAverageMiniAppOpenRate(t2DayAverageMiniAppOpenRate);
        double t2DayAverageT0FissionRate = item.getT2DayAverageT0FissionRate();
        if (data.getThirdFission0() > 0 && data.getSecondFirstLevel() > 0) {
            double t2DayT0FissionRate = data.getThirdFission0() / (double) data.getSecondFirstLevel();
            t2DayAverageT0FissionRate = (t2DayAverageT0FissionRate * (item.getActualArticleReleaseCount() - 1)
                    + t2DayT0FissionRate) / item.getActualArticleReleaseCount();
        }
        item.setT2DayAverageT0FissionRate(t2DayAverageT0FissionRate);
        double t2DayAverageFirstLayerFissionToViewBaseRatio = item.getT2DayAverageFirstLayerFissionToViewBaseRatio();
        if (data.getThirdFission0() > 0 && Objects.nonNull(data.getAvgViewCount()) && data.getAvgViewCount() > 0) {
            double t2DayFirstLayerFissionToViewBaseRatio = data.getThirdFission0() / data.getAvgViewCount();
            t2DayAverageFirstLayerFissionToViewBaseRatio = (t2DayAverageFirstLayerFissionToViewBaseRatio * (item.getActualArticleReleaseCount() - 1)
                    + t2DayFirstLayerFissionToViewBaseRatio) / item.getActualArticleReleaseCount();
        }
        item.setT2DayAverageFirstLayerFissionToViewBaseRatio(t2DayAverageFirstLayerFissionToViewBaseRatio);

        item.setT3DayViewCount(item.getT3DayViewCount());
        item.setT3DayFirstLayerUV(item.getT3DayFirstLayerUV());
        item.setT3DayT0FissionUV(item.getT3DayT0FissionUV());
        item.setT3DayAccountArticleRelevanceAvg(item.getT3DayAccountArticleRelevanceAvg());
        item.setT3DayAverageViewToBaseRatio(item.getT3DayAverageViewToBaseRatio());
        item.setT3DayAverageMiniAppOpenRate(item.getT3DayAverageMiniAppOpenRate());
        item.setT3DayAverageT0FissionRate(item.getT3DayAverageT0FissionRate());
        item.setT3DayAverageFirstLayerFissionToViewBaseRatio(item.getT3DayAverageFirstLayerFissionToViewBaseRatio());
        item.setT3DayViewBase(item.getT3DayViewBase());
        item.setT3PlusDayViewCount(item.getT3PlusDayViewCount());
        item.setT3PlusDayFirstLayerUV(item.getT3PlusDayFirstLayerUV());
        item.setT3PlusDayT0FissionUV(item.getT3PlusDayT0FissionUV());
        item.setT3PlusDayAccountArticleRelevanceAvg(item.getT3PlusDayAccountArticleRelevanceAvg());
        item.setT3PlusDayAverageViewToBaseRatio(item.getT3PlusDayAverageViewToBaseRatio());
        item.setT3PlusDayAverageMiniAppOpenRate(item.getT3PlusDayAverageMiniAppOpenRate());
        item.setT3PlusDayAverageT0FissionRate(item.getT3PlusDayAverageT0FissionRate());
        item.setT3PlusDayAverageFirstLayerFissionToViewBaseRatio(item.getT3PlusDayAverageFirstLayerFissionToViewBaseRatio());
        item.setT3PlusDayViewBase(item.getT3PlusDayViewBase());
    }

    private int getPromotionPoolLevel(String ghId, String title,
                                      Map<String, ArticlePoolPromotionSource> promotionSourceMap,
                                      List<String> small) {
        ArticlePoolPromotionSource promotionSource = promotionSourceMap.get(title);
        if (Objects.isNull(promotionSource)) {
            return 1;
        }
        if (promotionSource.getLevel().equals(ContentPoolEnum.autoArticlePoolLevel1.getContentPool())) {
            if (small.contains(ghId)) {
                return 3;
            } else {
                return 4;
            }
        } else if (promotionSource.getLevel().equals(ContentPoolEnum.autoArticlePoolLevel3.getContentPool())) {
            return 2;
        } else {
            return 1;
        }
    }

    private List<NewSortStrategyExport> getNewSortStrategyExportList(List<Article> todayPublish,
                                                                     Map<String, List<ArticleDetailInfo>> articleDetailInfoMap,
                                                                     Map<String, Map<String, Map<String, AccountAvgInfo>>> accountAvgInfoIndexMap) {
        List<NewSortStrategyExport> result = new ArrayList<>();
        for (Article article : todayPublish) {
            NewSortStrategyExport obj = new NewSortStrategyExport();
            List<ArticleDetailInfo> articleDetailInfos = articleDetailInfoMap.get(article.getWxSn());
            setObjArticleDetailInfo(article, obj, articleDetailInfos);
            AccountAvgInfo avgInfo = getAccountAvgInfo(accountAvgInfoIndexMap, article.getGhId(),
                    article.getPublishTimestamp(), article.getItemIndex());
            String date = DateUtils.timestampToYMDStr(article.getPublishTimestamp(), "yyyyMMdd");
            setObjBaseInfo(article, obj, date);
            setObjAvgInfo(article, obj, avgInfo);
            result.add(obj);
        }
        return result;
    }

    private void setPublishSourcePerformance(IntermediateIndicatorsExport item,
                                             NewSortStrategyExport data,
                                             Article article,
                                             Map<String, Map<String, Map<String, AccountAvgInfo>>> accountAvgInfoIndexMap,
                                             Map<String, List<ArticleDetailInfo>> articleDetailInfoMap,
                                             Map<String, Map<String, Map<Integer, List<PublishSortLog>>>> publishSortLogMap,
                                             String type,
                                             Map<String, Integer> scoreHisPublishTimeMap,
                                             Map<String, Set<String>> wxsnHisDistinctSetMap,
                                             List<Article> hisPublish,
                                             Integer poolLevel,
                                             List<String> small,
                                             Map<String, List<Article>> hisPublishMap) {
        Long hisMinDate = hisPublish.stream().mapToLong(Article::getPublishTimestamp).min().orElse(0);
        int scorePublishTime = scoreHisPublishTimeMap.computeIfAbsent(type, k -> 0);
        Set<String> wxsnHisDistinctSet = wxsnHisDistinctSetMap.computeIfAbsent(type, k -> new HashSet<>());
        for (Article publish : hisPublish) {
            if (poolLevel == 1) {
                List<Article> list = hisPublishMap.get(publish.getTitle());
                list = list.stream().filter(o -> o.getPublishTimestamp() < (publish.getPublishTimestamp() - 3600 * 8))
                        .collect(Collectors.toList());
                Integer hisPublishPoolLevel = getArticlePoolLevel(publish.getGhId(), list, small);
                if (hisPublishPoolLevel > poolLevel) {
                    continue;
                }
            }
            AccountAvgInfo avgInfo = getAccountAvgInfo(accountAvgInfoIndexMap, publish.getGhId(),
                    publish.getPublishTimestamp(), publish.getItemIndex());
            if (wxsnHisDistinctSet.contains(publish.getWxSn())) {
                continue;
            }
            wxsnHisDistinctSet.add(publish.getWxSn());
            item.setTotalArticleReleaseCountNonInfinite(item.getTotalArticleReleaseCountNonInfinite() + 1);
            item.setRedundantViewCount(item.getRedundantViewCount() + publish.getShowViewCount());
            if (Objects.nonNull(avgInfo)) {
                item.setRedundantViewBase(item.getRedundantViewBase() + avgInfo.getReadAvg());
            }
            if (item.getRedundantViewBase() > 0 && item.getRedundantViewCount() > 0) {
                item.setAverageViewToBaseRatio(item.getRedundantViewCount() / (double) item.getRedundantViewBase());
            }
            List<ArticleDetailInfo> articleDetailInfos = articleDetailInfoMap.get(publish.getWxSn());

            if (CollectionUtils.isEmpty(articleDetailInfos)) {
                continue;
            }
            int sumFirstLevel = 0;
            int sumFission0 = 0;
            Date minDate = articleDetailInfos.stream().map(ArticleDetailInfo::getRecallDt).min(Date::compareTo).orElse(new Date());
            for (ArticleDetailInfo articleDetailInfo : articleDetailInfos) {
                if (articleDetailInfo.getRecallDt().equals(minDate)) {
                    sumFirstLevel += Optional.ofNullable(articleDetailInfo.getFirstLevel()).orElse(0);
                    sumFission0 += Optional.ofNullable(articleDetailInfo.getFission0()).orElse(0);
                }
            }
            item.setRedundantFirstLayerUV(item.getRedundantFirstLayerUV() + sumFirstLevel);
            item.setRedundantT0FissionUV(item.getRedundantT0FissionUV() + sumFission0);
            double averageMiniAppOpenRate = item.getAverageMiniAppOpenRate();
            if (item.getRedundantFirstLayerUV() > 0 && item.getRedundantViewCount() > 0) {
                double miniAppOpenRate = item.getRedundantFirstLayerUV() / (double) item.getRedundantViewCount();
                averageMiniAppOpenRate = (averageMiniAppOpenRate * (item.getActualArticleReleaseCount() - 1)
                        + miniAppOpenRate) / item.getActualArticleReleaseCount();
            }
            item.setAverageMiniAppOpenRate(averageMiniAppOpenRate);
            double averageT0FissionRate = item.getAverageT0FissionRate();
            if (item.getRedundantT0FissionUV() > 0 && item.getRedundantFirstLayerUV() > 0) {
                double t0FissionRate = item.getRedundantT0FissionUV() / (double) item.getRedundantFirstLayerUV();
                averageT0FissionRate = (averageT0FissionRate * (item.getActualArticleReleaseCount() - 1)
                        + t0FissionRate) / item.getActualArticleReleaseCount();
            }
            item.setAverageT0FissionRate(averageT0FissionRate);
            double averageFirstLayerFissionToViewBaseRatio = item.getAverageFirstLayerFissionToViewBaseRatio();
            if (item.getRedundantViewBase() > 0 && item.getRedundantT0FissionUV() > 0) {
                double firstLayerFissionToViewBaseRatio = item.getRedundantT0FissionUV() / (double) item.getRedundantViewBase();
                averageFirstLayerFissionToViewBaseRatio = (averageFirstLayerFissionToViewBaseRatio *
                        (item.getActualArticleReleaseCount() - 1) + firstLayerFissionToViewBaseRatio) / item.getActualArticleReleaseCount();
            }
            item.setAverageFirstLayerFissionToViewBaseRatio(averageFirstLayerFissionToViewBaseRatio);
            Map<String, Map<Integer, List<PublishSortLog>>> dateSortMap = publishSortLogMap.get(publish.getGhId());
            if (Objects.nonNull(dateSortMap)) {
                String publishDateStr = DateUtils.timestampToYMDStr(publish.getPublishTimestamp(), "yyyyMMdd");
                Map<Integer, List<PublishSortLog>> indexMap = dateSortMap.get(publishDateStr);
                if (Objects.nonNull(indexMap)) {
                    List<PublishSortLog> logs = indexMap.get(publish.getItemIndex());
                    if (CollectionUtil.isNotEmpty(logs)) {
                        for (PublishSortLog log : logs) {
                            if (log.getTitle().equals(data.getTitle())) {
                                double scoreAvg = item.getRedundantAccountArticleRelevanceAvg();
                                double score = 0.0;
                                if (Objects.nonNull(log.getScore())) {
                                    JSONObject scoreMap = JSONObject.parseObject(log.getScoreMap());
                                    if (scoreMap.containsKey("SimilarityStrategy")) {
                                        score = scoreMap.getDoubleValue("SimilarityStrategy");
                                    }
                                }
                                scorePublishTime++;
                                scoreAvg = (scoreAvg * (scorePublishTime - 1) + score) / scorePublishTime;
                                item.setRedundantAccountArticleRelevanceAvg(scoreAvg);
                                break;
                            }
                        }
                    }
                }
            }
        }
        if (hisMinDate > 0) {
            double firstExplorationIntervalAvg = Double.isNaN(item.getFirstExplorationIntervalAvg()) ? 0.0 : item.getFirstExplorationIntervalAvg();
            double explorationInterval = (article.getPublishTimestamp() - hisMinDate) / 86400.0;
            firstExplorationIntervalAvg = (firstExplorationIntervalAvg * (item.getActualArticleReleaseCount() - 1)
                    + explorationInterval) / item.getActualArticleReleaseCount();
            item.setFirstExplorationIntervalAvg(firstExplorationIntervalAvg);
        }
        scoreHisPublishTimeMap.put(type, scorePublishTime);
    }

    private void setPublishPerformance(IntermediateIndicatorsExport item, NewSortStrategyExport data,
                                       Map<String, Map<String, Map<Integer, List<PublishSortLog>>>> publishSortLogMap) {
        if (Objects.isNull(data.getFirstLevel())) {
            return;
        }
        item.setViewCount(item.getViewCount() + data.getViewCount());
        item.setFirstLayerUV(item.getFirstLayerUV() + data.getFirstLevel());
        item.setT0FissionUV(item.getT0FissionUV() + data.getFission0());
        if (Objects.nonNull(data.getAvgViewCount())) {
            item.setViewBase(item.getViewBase() + data.getAvgViewCount());
        }
        if (item.getViewBase() > 0 && item.getViewCount() > 0) {
            item.setViewToBaseRatio(item.getViewCount() / (double) item.getViewBase());
        }
        if (item.getFirstLayerUV() > 0 && item.getViewCount() > 0) {
            item.setMiniAppOpenRate(item.getFirstLayerUV() / (double) item.getViewCount());
        }
        if (item.getFirstLayerUV() > 0 && item.getT0FissionUV() > 0) {
            item.setT0FissionRate(item.getT0FissionUV() / (double) item.getFirstLayerUV());
            item.setT0FissionRateAgain(item.getT0FissionUV() / (double) item.getFirstLayerUV());
        }
        item.setT0FissionCount(item.getT0FissionCount() + data.getFission0());
        item.setT1FissionCount(item.getT1FissionCount() + data.getFission1());
        item.setT2FissionCount(item.getT2FissionCount() + data.getFission2());
        if (item.getT0FissionCount() > 0 && item.getViewBase() > 0) {
            item.setFirstLayerFissionToViewBaseRatio(item.getT0FissionCount() / (double) item.getViewBase());
        }
        if (item.getT0FissionCount() + item.getT1FissionCount() > 0 && item.getFirstLayerUV() > 0) {
            item.setT1CumulativeFissionRate((item.getT0FissionCount() + item.getT1FissionCount()) / (double) item.getFirstLayerUV());
        }
        if (item.getT0FissionCount() + item.getT1FissionCount() + item.getT2FissionCount() > 0 && item.getViewBase() > 0) {
            item.setT2CumulativeFissionRate((item.getT0FissionCount() + item.getT1FissionCount() + item.getT2FissionCount()) / (double) item.getFirstLayerUV());
        }
        Map<String, Map<Integer, List<PublishSortLog>>> dateSortMap = publishSortLogMap.get(data.getGhId());
        if (Objects.nonNull(dateSortMap)) {
            Map<Integer, List<PublishSortLog>> indexMap = dateSortMap.get(item.getDateStr());
            if (Objects.nonNull(indexMap)) {
                List<PublishSortLog> logs = indexMap.get(data.getPosition());
                if (CollectionUtil.isNotEmpty(logs)) {
                    for (PublishSortLog log : logs) {
                        if (log.getTitle().equals(data.getTitle())) {
                            double scoreAvg = item.getAccountArticleRelevanceAvg();
                            double score = 0.0;
                            if (Objects.nonNull(log.getScore())) {
                                JSONObject scoreMap = JSONObject.parseObject(log.getScoreMap());
                                if (scoreMap.containsKey("SimilarityStrategy")) {
                                    score = scoreMap.getDoubleValue("SimilarityStrategy");
                                }
                            }
                            scoreAvg = (scoreAvg * (item.getActualArticleReleaseCount() - 1) + score) / item.getActualArticleReleaseCount();
                            item.setAccountArticleRelevanceAvg(scoreAvg);
                            break;
                        }
                    }
                }
            }
        }
    }

    private void setPublishSituation(IntermediateIndicatorsExport item, String type, Map<String, List<String>> titleTypeMap,
                                     Map<String, Map<Integer, List<String>>> titleTypePoolMap, Integer poolLevel,
                                     Article article, Map<String, List<String>> fansAccountTypeMap,
                                     AccountAvgInfo accountAvgInfo) {
        item.setArticleReleaseSlot(item.getArticleReleaseSlot() + 1);
        item.setActualArticleReleaseCount(item.getActualArticleReleaseCount() + 1);
        if (titleTypeMap.containsKey(type) && !titleTypeMap.get(type).contains(article.getTitle())) {
            item.setDistinctArticleReleaseCount(item.getDistinctArticleReleaseCount() + 1);
        }
        item.setDistinctArticleReleaseRatio(item.getDistinctArticleReleaseCount() / (double) item.getActualArticleReleaseCount());
        List<String> ghIds = fansAccountTypeMap.computeIfAbsent(type, k -> new ArrayList<>());
        if (!ghIds.contains(article.getGhId())) {
            ghIds.add(article.getGhId());
            if (Objects.nonNull(accountAvgInfo)) {
                item.setFansCount(item.getFansCount() + accountAvgInfo.getFans());
            }
        }
        if (4 == poolLevel) {
            item.setActualReleaseCountL4(item.getActualReleaseCountL4() + 1);
        } else if (3 == poolLevel) {
            item.setActualReleaseCountL3(item.getActualReleaseCountL3() + 1);
        } else if (2 == poolLevel) {
            item.setActualReleaseCountL2(item.getActualReleaseCountL2() + 1);
        } else {
            item.setActualReleaseCountL1(item.getActualReleaseCountL1() + 1);
        }
        if (titleTypePoolMap.containsKey(type)
                && titleTypePoolMap.get(type).containsKey(poolLevel)
                && !titleTypePoolMap.get(type).get(poolLevel).contains(article.getTitle())) {
            if (4 == poolLevel) {
                item.setDistinctReleaseCountL4(item.getDistinctReleaseCountL4() + 1);
            } else if (3 == poolLevel) {
                item.setDistinctReleaseCountL3(item.getDistinctReleaseCountL3() + 1);
            } else if (2 == poolLevel) {
                item.setDistinctReleaseCountL2(item.getDistinctReleaseCountL2() + 1);
            } else {
                item.setDistinctReleaseCountL1(item.getDistinctReleaseCountL1() + 1);
            }
        }
        if (item.getActualReleaseCountL1() > 0 && item.getDistinctReleaseCountL1() > 0) {
            item.setDistinctToActualRatioL1(item.getDistinctReleaseCountL1() / (double) item.getActualReleaseCountL1());
        }
        if (item.getActualReleaseCountL2() > 0 && item.getDistinctReleaseCountL2() > 0) {
            item.setDistinctToActualRatioL2(item.getDistinctReleaseCountL2() / (double) item.getActualReleaseCountL2());
        }
        if (item.getActualReleaseCountL3() > 0 && item.getDistinctReleaseCountL3() > 0) {
            item.setDistinctToActualRatioL3(item.getDistinctReleaseCountL3() / (double) item.getActualReleaseCountL3());
        }
        if (item.getActualReleaseCountL4() > 0 && item.getDistinctReleaseCountL4() > 0) {
            item.setDistinctToActualRatioL4(item.getDistinctReleaseCountL4() / (double) item.getActualReleaseCountL4());
        }
    }

    private Integer getArticlePoolLevel(String ghId, List<Article> list, List<String> small) {
        int minPosition = list.stream().mapToInt(Article::getItemIndex).min().orElse(8);
        if (minPosition == 1) {
            if (small.contains(ghId)) {
                return 3;
            } else {
                return 4;
            }
        } else if (minPosition == 2) {
            return 2;
        } else {
            return 1;
        }
    }

    private String getArticleType(NewSortStrategyExport data, List<String> small) {
        if (data.getPosition() == 1) {
            if (small.contains(data.getGhId())) {
                return "L3";
            } else {
                return "L4";
            }
        } else if (data.getPosition() == 2) {
            return "L2";
        } else {
            return "L1";
        }
    }

    private List<IntermediateIndicatorsExport> buildIntermediateIndicatorsData(String dateStr) {
        List<IntermediateIndicatorsExport> result = new ArrayList<>();
        List<String> typeList = Arrays.asList("L4", "L3", "L2", "L1");
        for (String type : typeList) {
            IntermediateIndicatorsExport item = new IntermediateIndicatorsExport();
            item.setDateStr(dateStr);
            item.setType(type);
            result.add(item);
        }
        return result;
    }

    @XxlJob("scheduleExportFirstContentScore")
    public ReturnT<String> scheduleExportFirstContentScore(String param) {
        List<String> dateStrList = DateUtils.getBeforeDays(null, null, 1);
        exportFeishuFirstContentScore(dateStrList, sheetToken, "XBFd16");
        return ReturnT.SUCCESS;
    }

    public void firstContentScoreExport(String beginDate, String endDate) {
        List<String> dateStrList = DateUtils.getBeforeDays(beginDate, endDate, 3);
        exportFeishuFirstContentScore(dateStrList, sheetToken, "XBFd16");
    }

    private void exportFeishuFirstContentScore(List<String> dateStrList, String sheetToken, String sheetId) {
        List<FirstContentScoreExport> scoreDataList = firstContentScoreData(dateStrList);
        if (CollectionUtil.isEmpty(scoreDataList)) {
            return;
        }
        int rowNum = scoreDataList.size();
        List<List<Object>> rows = new ArrayList<>();
        Field[] fields = FirstContentScoreExport.class.getDeclaredFields();
        for (FirstContentScoreExport datum : scoreDataList) {
            List<Object> rowDatas = new ArrayList<>();
            rows.add(rowDatas);

            for (Field field : fields) {
                field.setAccessible(true);
                try {
                    rowDatas.add(field.get(datum));
                } catch (IllegalAccessException e) {
                    log.error("获取值出错：{}", field.getName());
                } catch (Exception e) {
                    throw new RuntimeException(e.getMessage());
                }
            }
        }

        List<Pair<String, String>> styles = Arrays
                .asList(
                        Pair.of("H", "#,##0.00"),
                        Pair.of("I", "#,##0.00"),
                        Pair.of("J", "#,##0.00"),
                        Pair.of("K", "#,##0.00"),
                        Pair.of("L", "#,##0.00"),
                        Pair.of("M", "#,##0.00"),
                        Pair.of("N", "#,##0.00"),
                        Pair.of("O", "#,##0.00"),
                        Pair.of("P", "#,##0.00"),
                        Pair.of("Q", "#,##0.00"),
                        Pair.of("R", "#,##0.00"),
                        Pair.of("S", "#,##0.00"),
                        Pair.of("T", "#,##0.00"),
                        Pair.of("W", "#,##0.00")
                );

        doSendFeishuSheet(dateStrList, sheetToken, sheetId, rowNum, rows, 2, styles);
    }

    private List<FirstContentScoreExport> firstContentScoreData(List<String> dateStrList) {
        List<FirstContentScoreExport> result = new ArrayList<>();
        List<PublishSortLog> sortLogList = publishSortLogRepository.findByDateStrIn(dateStrList);
        sortLogList = sortLogList.stream().filter(o -> o.getIndex() == 1 || o.getIndex() == 2).collect(Collectors.toList());
        sortLogList.sort(Comparator.comparing(PublishSortLog::getGhId).thenComparing(PublishSortLog::getDateStr));
        List<String> ghIds = sortLogList.stream().map(PublishSortLog::getGhId).distinct().collect(Collectors.toList());
        long minTimestamp = DateUtils.dateStrToTimestamp(dateStrList.get(0), "yyyyMMdd");
        List<Article> articleList = articleRepository.getByGhIdInAndPublishTimestampGreaterThanAndTypeEquals(ghIds, minTimestamp, ArticleTypeEnum.QUNFA.getVal());
        articleList = articleList.stream().filter(o -> o.getItemIndex() == 1 || o.getItemIndex() == 2).collect(Collectors.toList());
        Map<String, Map<String, Map<Integer, Article>>> articleMap = articleList.stream().collect(
                Collectors.groupingBy(Article::getGhId, Collectors.groupingBy(o -> DateUtils.timestampToYMDStr(o.getPublishTimestamp(), "yyyyMMdd"),
                        Collectors.toMap(Article::getItemIndex, o -> o,
                                (existing, replacement) -> replacement))));
        List<String> titleList = articleList.stream().map(Article::getTitle).distinct().collect(Collectors.toList());
        List<PublishContentDTO> hisPublishList = new ArrayList<>();
        for (List<String> partitions : Lists.partition(new ArrayList<>(titleList), 100)) {
            hisPublishList.addAll(aigcBaseMapper.getHisPublishByTitles(partitions));
        }
        Map<String, List<PublishContentDTO>> hisPublishMap = hisPublishList.stream().collect(Collectors.groupingBy(PublishContentDTO::getTitle));
        String ymd = DateUtils.timestampToYMDStr(minTimestamp - 86400 * 7, "yyyy-MM-dd");
        List<AccountAvgInfo> accountAvgInfoList = accountAvgInfoRepository.getAllByUpdateTimeGreaterThanEqual(ymd);
        Map<String, Map<String, Map<String, AccountAvgInfo>>> accountAvgInfoMap = accountAvgInfoList.stream()
                .filter(o -> Objects.equals(o.getPosition(), "1") || Objects.equals(o.getPosition(), "2"))
                .collect(Collectors.groupingBy(AccountAvgInfo::getGhId,
                        Collectors.groupingBy(AccountAvgInfo::getUpdateTime,
                                Collectors.toMap(AccountAvgInfo::getPosition, o -> o))));

        String title = "";
        List<DatastatScore> saveList = new ArrayList<>();
        for (PublishSortLog publishSortLog : sortLogList) {
            Map<String, Map<Integer, Article>> dateArticleMap = articleMap.get(publishSortLog.getGhId());
            if (Objects.isNull(dateArticleMap)) {
                continue;
            }
            Map<Integer, Article> indexMap = dateArticleMap.get(publishSortLog.getDateStr());
            if (Objects.isNull(indexMap)) {
                continue;
            }
            Article article = indexMap.get(publishSortLog.getIndex());
            if (Objects.isNull(article) || !publishSortLog.getTitle().equals(article.getTitle())) {
                continue;
            }
            if (publishSortLog.getTitle().equals(title)) {
                continue;
            }
            FirstContentScoreExport item = new FirstContentScoreExport();
            title = publishSortLog.getTitle();
            item.setDateStr(publishSortLog.getDateStr());
            item.setGhId(publishSortLog.getGhId());
            item.setAccountName(publishSortLog.getAccountName());
            item.setIndex(publishSortLog.getIndex());
            item.setTitle(publishSortLog.getTitle());
            if (StringUtils.hasText(publishSortLog.getCategory())) {
                List<String> categoryList = JSONArray.parseArray(publishSortLog.getCategory(), String.class);
                item.setCategory(categoryList.get(0));
            }
            item.setStrategy(publishSortLog.getStrategy());
            item.setScore(Double.valueOf(publishSortLog.getScore()));
            JSONObject scoreMap = JSONObject.parseObject(publishSortLog.getScoreMap());
            item.setHisFissionFansRateRateStrategy(scoreMap.getDoubleValue("HisFissionFansRateRateStrategy"));
            item.setHisFissionAvgReadRateRateStrategy(scoreMap.getDoubleValue("HisFissionAvgReadRateRateStrategy"));
            item.setPublishTimesStrategy(scoreMap.getDoubleValue("PublishTimesStrategy"));
            item.setViewCountRateCorrelationStrategy(scoreMap.getDoubleValue("ViewCountRateCorrelationStrategy"));
            item.setHisFissionAvgReadSumRateStrategy(scoreMap.getDoubleValue("HisFissionAvgReadSumRateStrategy"));
            item.setHisFissionAvgReadRateCorrelationRateStrategy(scoreMap.getDoubleValue("HisFissionAvgReadRateCorrelationRateStrategy"));
            item.setHisFissionFansSumRateStrategy(scoreMap.getDoubleValue("HisFissionFansSumRateStrategy"));
            item.setSimilarityStrategy(scoreMap.getDoubleValue("SimilarityStrategy"));
            item.setCategoryStrategy(scoreMap.getDoubleValue("CategoryStrategy"));
            item.setViewCountStrategy(scoreMap.getDoubleValue("ViewCountStrategy"));
            item.setViewCountRateStrategy(scoreMap.getDoubleValue("ViewCountRateStrategy"));
            item.setHisFissionDeWeightAvgReadSumRateStrategy(scoreMap.getDoubleValue("HisFissionDeWeightAvgReadSumRateStrategy"));
            item.setReadCount(article.getShowViewCount());
            Map<String, Map<String, AccountAvgInfo>> map = accountAvgInfoMap.get(article.getGhId());
            if (Objects.nonNull(map)) {
                List<String> avgMapDateList = new ArrayList<>(map.keySet());
                String publishDate = DateUtils.findNearestDate(avgMapDateList,
                        DateUtils.timestampToYMDStr(article.getPublishTimestamp(), "yyyy-MM-dd"), "yyyy-MM-dd");
                Map<String, AccountAvgInfo> avgIndexMap = map.get(publishDate);
                if (Objects.nonNull(avgIndexMap)) {
                    AccountAvgInfo accountAvgInfo = avgIndexMap.get(String.valueOf(publishSortLog.getIndex()));
                    if (Objects.nonNull(accountAvgInfo)) {
                        item.setReadAvg(accountAvgInfo.getReadAvg());
                        item.setReadAvgRate(article.getShowViewCount() / (double) accountAvgInfo.getReadAvg());
                    }
                }
            }
            List<PublishContentDTO> hisPublish = hisPublishMap.get(article.getTitle());
            if (CollectionUtils.isNotEmpty(hisPublish)) {
                long hisMinDate = hisPublish.stream().filter(o -> Objects.nonNull(o.getPublishTimestamp()))
                        .mapToLong(PublishContentDTO::getPublishTimestamp).min().orElse(0);
                int explorationInterval = (int) ((article.getPublishTimestamp() - (hisMinDate / 1000)) / 86400);
                item.setFirstExplorationIntervalAvg(explorationInterval);
            }
            result.add(item);
            saveList.add(buildDatastatScore(publishSortLog, item));
        }
        saveDatastatScore(dateStrList, saveList);
        result = result.stream().filter(o -> o.getIndex() == 1).collect(Collectors.toList());
        result.sort(Comparator.comparing(FirstContentScoreExport::getDateStr).reversed()
                .thenComparing(FirstContentScoreExport::getGhId));
        return result;
    }

    private DatastatScore buildDatastatScore(PublishSortLog publishSortLog, FirstContentScoreExport value) {
        DatastatScore item = new DatastatScore();
        item.setDt(value.getDateStr());
        item.setGhId(value.getGhId());
        item.setAccountName(value.getAccountName());
        item.setIndex(value.getIndex());
        item.setTitle(value.getTitle());
        item.setCategory(value.getCategory());
        item.setStrategy(value.getStrategy());
        item.setScore(value.getScore());
        item.setHisFissionAvgReadRateRate(value.getHisFissionAvgReadRateRateStrategy());
        item.setHisFissionAvgReadSumRate(value.getHisFissionAvgReadSumRateStrategy());
        item.setSimilarity(value.getSimilarityStrategy());
        item.setCategoryScore(value.getCategoryStrategy());
        item.setViewCountRate(value.getViewCountRateStrategy());
        item.setHisFissionDeWeightAvgReadSumRate(value.getHisFissionDeWeightAvgReadSumRateStrategy());
        item.setReadCount(value.getReadCount());
        item.setReadAvg(value.getReadAvg());
        item.setReadAvgRate(value.getReadAvgRate());
        item.setFirstPubInterval(value.getFirstExplorationIntervalAvg());
        item.setPublishContentId(publishSortLog.getPublishContentId());
        item.setCrawlerChannelContentId(publishSortLog.getCrawlerChannelContentId());
        item.setSourceId(publishSortLog.getSourceId());
        return item;
    }

    private void saveDatastatScore(List<String> dateStrList, List<DatastatScore> saveList) {
        if (CollectionUtils.isNotEmpty(saveList)) {
            longArticleBaseMapper.deleteDatastatScoreByDtIn(dateStrList);
            for (List<DatastatScore> saveListPartition : Lists.partition(saveList, 1000)) {
                longArticleBaseMapper.batchInsertDatastatScore(saveListPartition);
            }
        }
    }

}
